#include "Navigation/libraries/client/ComponentPlugin.h"

#include "ArmarXCore/core/Component.h"
#include "ArmarXCore/core/application/properties/PropertyDefinitionContainer.h"
#include "ArmarXCore/util/CPPUtility/trace.h"

#include "Navigation/libraries/client/Navigator.h"

// ComponentPlugin

armarx::nav::client::ComponentPlugin::~ComponentPlugin() = default;

void
armarx::nav::client::ComponentPlugin::postCreatePropertyDefinitions(
    armarx::PropertyDefinitionsPtr& properties)
{
    if (not properties->hasDefinition(PROPERTY_NAME))
    {
        properties->defineOptionalProperty<std::string>(
            PROPERTY_NAME, "Navigator", "Name of the Navigator");
    }
}

void
armarx::nav::client::ComponentPlugin::preOnInitComponent()
{
    ARMARX_TRACE;
    parent<armarx::Component>().usingProxyFromProperty(PROPERTY_NAME);
}

void
armarx::nav::client::ComponentPlugin::preOnConnectComponent()
{
    ARMARX_TRACE;
    parent<armarx::Component>().getProxyFromProperty(navigatorPrx, PROPERTY_NAME);

    ARMARX_TRACE;
    const std::string componentName = parent().getName();
    IceNavigator& srv = parent<armarx::nav::client::ComponentPluginUser>().iceNavigator;
    srv.setNavigatorComponent(navigatorPrx);
}

// ComponentPluginUser

armarx::nav::client::ComponentPluginUser::ComponentPluginUser() : navigator{navigatorServices}
{
    ARMARX_TRACE;
    addPlugin(plugin);
}

void
armarx::nav::client::ComponentPluginUser::configureNavigator(
    const client::NavigationStackConfig& stackConfig)
{
    configureNavigator(stackConfig, getName());
}

void
armarx::nav::client::ComponentPluginUser::configureNavigator(
    const client::NavigationStackConfig& stackConfig,
    const std::string& configId)
{
    iceNavigator.createConfig(stackConfig, configId);
}

armarx::nav::client::ComponentPluginUser::~ComponentPluginUser() = default;
