#pragma once

// ArmarX
#include <ArmarXCore/core/ComponentPlugin.h>
#include <ArmarXCore/core/ManagedIceObject.h>

// Navigator
#include <Navigation/components/Navigator/NavigatorInterface.h>
#include <Navigation/libraries/client/Navigator.h>
#include <Navigation/libraries/client/services/IceNavigator.h>
#include <Navigation/libraries/client/services/MemorySubscriber.h>

namespace armarx::nav::client
{

    class ComponentPlugin : virtual public armarx::ComponentPlugin
    {

    public:
        using armarx::ComponentPlugin::ComponentPlugin;

        ~ComponentPlugin() override;

        void postCreatePropertyDefinitions(armarx::PropertyDefinitionsPtr& properties) override;

        void preOnInitComponent() override;

        void preOnConnectComponent() override;

    private:
        static constexpr const char* PROPERTY_NAME = "nav.NavigatorName";

        components::NavigatorInterfacePrx navigatorPrx;
    };

    class ComponentPluginUser : virtual public ManagedIceObject
    {

    public:
        ComponentPluginUser();

        void configureNavigator(const client::NavigationStackConfig& stackConfig);

        void configureNavigator(const client::NavigationStackConfig& stackConfig,
                                const std::string& configId);

        Navigator navigator;

        // Non-API

    public:
        ~ComponentPluginUser() override;

        friend class ComponentPlugin;

    private:
        IceNavigator iceNavigator;
        MemorySubscriber memorySubscriber;
        Navigator::InjectedServices navigatorServices{.navigator = &iceNavigator,
                                                      .subscriber = &memorySubscriber};

        ComponentPlugin* plugin = nullptr;
    };

} // namespace armarx::nav::client
