/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    Navigation::ArmarXObjects::factories
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @author     Christian R. G. Dreher ( c dot dreher at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "Navigation/libraries/client/NavigationStackConfig.h"
#include "Navigation/libraries/core/types.h"
#include "Navigation/libraries/global_planning/AStar.h"
#include "Navigation/libraries/trajectory_control/TrajectoryFollowingController.h"
#define BOOST_TEST_MODULE Navigation::ArmarXLibraries::factories

#define ARMARX_BOOST_TEST

#include "../NavigationStackFactory.h"
#include <Navigation/Test.h>


BOOST_AUTO_TEST_CASE(testGlobalPlannerCreation)
{
    // armarx::nav::client::NavigationStackConfig cfg;
    // cfg.globalPlanner(armarx::nav::glob_plan::AStarParams())
    // .trajectoryController(armarx::nav::traj_ctrl::TrajectoryFollowingController::Params());

    // const auto dto = cfg.toAron();

    // armarx::nav::core::Scene scene; // TODO mock

    // const auto navStack = armarx::nav::fac::NavigationStackFactory::create(dto, scene);

    // BOOST_CHECK(dynamic_cast<armarx::nav::glob_plan::AStar*>(navStack.globalPlanner.get()) != nullptr);
    // BOOST_CHECK(dynamic_cast<armarx::nav::traj_ctrl::TrajectoryFollowingController*>(navStack.trajectoryControl.get()) != nullptr);
}


BOOST_AUTO_TEST_CASE(testStackCreation)
{
    armarx::nav::client::NavigationStackConfig cfg;
    cfg.globalPlanner(armarx::nav::glob_plan::AStarParams())
        .trajectoryController(armarx::nav::traj_ctrl::TrajectoryFollowingController::Params());

    const auto dto = cfg.toAron();

    armarx::nav::core::Scene scene; // TODO mock

    const auto navStack = armarx::nav::fac::NavigationStackFactory::create(dto, scene);

    BOOST_CHECK(navStack.globalPlanner.get() != nullptr);
    BOOST_CHECK(navStack.trajectoryControl.get() != nullptr);

    BOOST_CHECK(dynamic_cast<armarx::nav::glob_plan::AStar*>(navStack.globalPlanner.get()) !=
                nullptr);
    BOOST_CHECK(dynamic_cast<armarx::nav::traj_ctrl::TrajectoryFollowingController*>(
                    navStack.trajectoryControl.get()) != nullptr);
}
